package live.exit.musicplayer

import android.R
import android.app.*
import android.content.Intent
import android.os.IBinder
import android.content.Context
import android.app.PendingIntent
import android.app.NotificationManager
import android.graphics.Bitmap
import android.graphics.drawable.Icon
import android.media.session.MediaSession

import android.os.Binder
import android.util.Log

class MusicPlayerService : Service() {

  private var notificationManager: NotificationManager? = null
  private var notificationData: NotificationData = NotificationData()
  private var session: MediaSession? = null

  private val play = Intent(NOTIFY_PLAY)
  private val pause = Intent(NOTIFY_PAUSE)
  private val previous = Intent(NOTIFY_PREVIOUS)
  private val next = Intent(NOTIFY_NEXT)

  private var playAction: Notification.Action? = null
  private var pauseAction: Notification.Action? = null
  private var previousAction: Notification.Action? = null
  private var nextAction: Notification.Action? = null

  private val notificationId = 85711
  private val channelId = "music_player_notifications"
  private val channelName = "Music Player Notification Channel"
  private val binder = LocalBinder()

  inner class LocalBinder : Binder() {
    val service: MusicPlayerService
      get() = this@MusicPlayerService
  }

  override fun onBind(intent: Intent?): IBinder? {
    Log.v("MusicPlayerService", "Bound!")
    return binder
  }

  override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
    Log.v("MusicPlayerService", "Starting...")
    val channel = NotificationChannel(
        channelId,
        channelName, NotificationManager.IMPORTANCE_LOW
    )
    channel.setSound(null, null)
    channel.enableLights(false)
    channel.lockscreenVisibility = Notification.VISIBILITY_PUBLIC
    val service = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
    service.createNotificationChannel(channel)
    startForeground(notificationId, updateNotification(notificationData))
    notificationManager = applicationContext.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
    session = MediaSession(applicationContext, "MediaSession")
    val playIcon = Icon.createWithResource(applicationContext, R.drawable.ic_media_play)
    val pauseIcon = Icon.createWithResource(applicationContext, R.drawable.ic_media_pause)
    val previousIcon = Icon.createWithResource(applicationContext, R.drawable.ic_media_previous)
    val nextIcon = Icon.createWithResource(applicationContext, R.drawable.ic_media_next)
    playAction = Notification.Action.Builder(playIcon, "Play", PendingIntent.getBroadcast(applicationContext, 1, play, PendingIntent.FLAG_UPDATE_CURRENT)).build()
    pauseAction = Notification.Action.Builder(pauseIcon, "Pause", PendingIntent.getBroadcast(applicationContext, 1, pause, PendingIntent.FLAG_UPDATE_CURRENT)).build()
    previousAction = Notification.Action.Builder(previousIcon, "Previous", PendingIntent.getBroadcast(applicationContext, 1, previous, PendingIntent.FLAG_UPDATE_CURRENT)).build()
    nextAction = Notification.Action.Builder(nextIcon, "Next", PendingIntent.getBroadcast(applicationContext, 1, next, PendingIntent.FLAG_UPDATE_CURRENT)).build()
    return START_NOT_STICKY
  }

  fun updateNotification(notificationData: NotificationData): Notification {
    Log.v("MusicPlayerService", "Updating Notification...")
    this.notificationData = notificationData

    val builder = Notification.Builder(applicationContext, channelId)
        .setSmallIcon(notificationData.notificationIcon
            ?: Icon.createWithResource(applicationContext, R.drawable.ic_media_play))
        .setContentTitle(notificationData.trackName)
        .setContentText("${notificationData.artistName} - ${notificationData.albumName}")
        .setLargeIcon(notificationData.coverImage)
        .setStyle(Notification.MediaStyle().setMediaSession(session?.sessionToken))
        .setOngoing(true)
    builder.addAction(previousAction)
    if (notificationData.isPlaying) {
      builder.addAction(pauseAction)
    } else {
      builder.addAction(playAction)
    }
    builder.addAction(nextAction)

    val notification = builder.build()
    notificationManager?.notify(notificationId, notification)
    return notification
  }
}

class NotificationData {
  var isPlaying = true
  var trackName: String? = ""
  var albumName: String? = ""
  var artistName: String? = ""
  var coverImage: Bitmap? = null
  var notificationIcon: Icon? = null
}